@@ -24,14 +24,18 @@ SOFTWARE.
 import json
 import logging
 import threading
-
+import time
 import shortuuid
 import zmq
 
-from .constants import DEVICE_PUB_URL, BAD_CHARS
+from .constants import CONNECTION_PUB_URL, BAD_CHARS
+from .action_station import ActionStation
 from .iotcontrol.alarm import Alarm
 from .iotcontrol.device_view import DeviceView
+from .iotcontrol.enums import ControlName
 from .load_config import encode_cfg64
+from .load_config import CONTROL_INSTANCE_DICT, CONFIG_INSTANCE_DICT
+
 
 class Device(threading.Thread):
     """Dashio Device
@@ -50,6 +54,15 @@ class Device(threading.Thread):
     add_control(iot_control) :
         Add a control to the device.
 
+    remove_control(iot_control) :
+        Remove a control from the device.
+
+    get_control(control_type: str, control_id: str):
+        returns the instance of a controla leaded into the device.
+
+    add_all_c64_controls(c64_dict: dict):
+        Adds all controls defined in c64_dict into the device.
+
     set_wifi_callback(callback) :
         Set a callback function that is called when the DashIO app provides wifi provisioning information.
 
@@ -108,7 +121,7 @@ class Device(threading.Thread):
         if cntrl_type in self._device_commands_dict:
             return self._device_commands_dict[cntrl_type](data_array)
         try:
-            reply = self._control_dict[cntrl_type + "_" + data_array[2]].message_rx_event(data_array)
+            reply = self.controls_dict[cntrl_type + "\t" + data_array[2]]._message_rx_event(data_array)
             if reply:
                 return reply.replace("{device_id}", self.device_id)
         except (KeyError, IndexError):
@@ -120,7 +133,7 @@ class Device(threading.Thread):
 
     def _make_status(self, _):
         reply = f"\t{self.device_id}\tNAME\t{self._device_name}\n"
-        for value in self._control_dict.values():
+        for value in self.controls_dict.values():
             try:
                 reply += value.get_state().format(device_id=self.device_id)
             except (TypeError, KeyError):
@@ -136,15 +149,15 @@ class Device(threading.Thread):
         reply = self._device_id_str + f"\tCFG\t{dashboard_id}\tC64\t"
         cfg = {}
         cfg["CFG"] = self._cfg
-        for control in self._control_dict.values():
+        for control in self.controls_dict.values():
             if control.cntrl_type == "ALM":
                 continue
-            if control.cntrl_type in ("TCP", "MQTT", "BLE"):
+            if control.cntrl_type in ("BLE"):
                 cfg[control.cntrl_type] = control.get_cfg64(data)
                 continue
             if control.cntrl_type not in cfg:
                 cfg[control.cntrl_type] = []
-            cfg[control.cntrl_type].append(control.get_cfg64(data))
+            cfg[control.cntrl_type].extend(control.get_cfg64(data))
         c64_json = encode_cfg64(cfg)
         reply += c64_json + "\n"
         return reply
@@ -155,36 +168,68 @@ class Device(threading.Thread):
             #  no_views = data[3]
         except IndexError:
             return ""
-        reply = self._device_id_str + f"\tCFG\t{dashboard_id}\tDVCE\t" + json.dumps(self._cfg) + "\n"
+        reply = self._device_id_str + f"\tCFG\t{dashboard_id}\tDVCE\t{json.dumps(self._cfg)}\n"
         dvvw_str = ""
-        for control in self._control_dict.values():
+        for control in self.controls_dict.values():
             if control.cntrl_type == "ALM":
                 continue
             if control.cntrl_type == "DVVW":
-                dvvw_str += self._device_id_str + control.get_cfg(data)
+                cfg_list = control.get_cfg(data)
+                for cfg in cfg_list:
+                    dvvw_str += self._device_id_str + cfg
             else:
-                reply += self._device_id_str + control.get_cfg(data)
+                cfg_list = control.get_cfg(data)
+                #  logging.debug("CFG: %s", cfg_list)
+                for cfg in cfg_list:
+                    reply += self._device_id_str + cfg
         reply += dvvw_str
         return reply
 
     def _send_alarm(self, alarm_id, message_header, message_body):
-        payload = self._device_id_str + f"\t{alarm_id}\t{message_header}\t{message_body}\n"
+        payload = self._device_id_str + f"\tALM\t{alarm_id}\t{message_header}\t{message_body}\n"
         logging.debug("ALARM: %s", payload)
-        self.tx_zmq_pub.send_multipart([b"ALARM", b'0', payload.encode('utf-8')])
+        self.tx_zmq_pub.send_multipart([b"ALL", payload.encode('utf-8')])
 
     def _send_data(self, data: str):
         if not data:
             return
-        reply_send = data.format(device_id=self.device_id)
+        if isinstance(data, str):
+            reply_send = data.format(device_id=self.device_id)
+        elif isinstance(data, list):
+            reply_send = "\t" + "\t".join(data) + "\n"
         try:
-            self.tx_zmq_pub.send_multipart([b"ALL", b'0', reply_send.encode('utf-8')])
+            self.tx_zmq_pub.send_multipart([b"ALL", reply_send.encode('utf-8')])
         except zmq.error.ZMQError:
             pass
 
     def _send_announce(self):
         payload = self._device_id_str + f"\tWHO\t{self.device_type}\t{self.device_name}\n"
         logging.debug("ANNOUNCE: %s", payload)
-        self.tx_zmq_pub.send_multipart([b"ANNOUNCE", b'0', payload.encode('utf-8')])
+        self.tx_zmq_pub.send_multipart([b"DASH", payload.encode('utf-8')])
+
+    def is_control_loaded(self, control_type, control_id) -> bool:
+        """Is the control loaded in the device?"""
+        key = f"{control_type}\t{control_id}"
+        return key in self.controls_dict
+
+    def add_all_c64_controls(self, c64_dict):
+        """Loads all the controls in cfg_dict into the device.
+
+        Parameters
+        ----------
+        c64_dict : Dict
+            dictionary of the CFG loaded by decode_cfg from a CFG64 or json
+        """
+        for control_type, control_list in c64_dict.items():
+            if isinstance(control_list, list):
+                for control in control_list:
+                    key = f"{control_type}\t{control['controlID']}"
+                    if self.is_control_loaded(control_type, control['controlID']):
+                        cfg = CONFIG_INSTANCE_DICT[control_type].from_dict(control)
+                        self.controls_dict[key].add_config_columnar(cfg)
+                    else:
+                        new_control = CONTROL_INSTANCE_DICT[control_type].from_cfg_dict(control)
+                        self.add_control(new_control)
 
     def add_control(self, iot_control):
         """Add a control to the device.
@@ -193,22 +238,54 @@ class Device(threading.Thread):
         ----------
             iot_control : iotControl
         """
-        if isinstance(iot_control, DeviceView):
-            self._cfg["numDeviceViews"] += 1
         try:
             if isinstance(iot_control, Alarm):
-                iot_control.message_tx_event += self._send_alarm
+                iot_control.add_transmit_message_callback(self._send_alarm)
             else:
-                iot_control.message_tx_event += self._send_data
+                iot_control.add_transmit_message_callback(self._send_data)
         except AttributeError:
             pass
-        key = iot_control.cntrl_type + "_" + iot_control.control_id
-        self._control_dict[key] = iot_control
+        key = f"{iot_control.cntrl_type}\t{iot_control.control_id}"
+
+        if key not in self.controls_dict:
+            self.inc_config_revision()
+            if isinstance(iot_control, DeviceView):
+                self._cfg["numDeviceViews"] += 1
+            self.controls_dict[key] = iot_control
+            return True
+        return False
+
+    def get_control(self, control_type: ControlName, control_id: str):
+        """Get the control instance.
+
+        Parameters
+        ----------
+            control_type : ControlName
+                The type of control
+            control_id : str
+                The controlID of the control
+        """
+        key = f"{control_type.value}\t{control_id}"
+        return self.controls_dict.get(key, None)
+
+    def remove_control(self, iot_control):
+        """Remove a control a control to the device.
+
+        Parameters
+        ----------
+            iot_control : iotControl
+        """
+        if isinstance(iot_control, DeviceView):
+            self._cfg["numDeviceViews"] -= 1
+        key = f"{iot_control.cntrl_type}\t{iot_control.control_id}"
+        if key in self.controls_dict:
+            del self.controls_dict[key]
 
     def _set_devicesetup(self, control_name: str, settable: bool):
         if settable:
             self._device_commands_dict[control_name.upper()] = getattr(self, '_' + control_name + '_rx_event', None)
-            self._device_setup_list.append(control_name)
+            if control_name not in self._device_setup_list:
+                self._device_setup_list.append(control_name)
         else:
             try:
                 del self._device_commands_dict[control_name.upper()]
@@ -220,6 +297,21 @@ class Device(threading.Thread):
                 pass
         self._cfg["deviceSetup"] = ','.join(self._device_setup_list)
 
+    def _add_action_device_setup(self, settable: bool):
+        if settable:
+            if 'actn' not in self._device_setup_list:
+                self._device_setup_list.append('actn')
+        else:
+            try:
+                del self._device_commands_dict['actn']
+            except KeyError:
+                pass
+            try:
+                self._device_setup_list.remove('actn')
+            except ValueError:
+                pass
+        self._cfg["deviceSetup"] = ','.join(self._device_setup_list)
+
     def set_wifi_callback(self, callback):
         """
         Specify a callback function to be called when IoTDashboard sets wifi parameters.
@@ -243,7 +335,7 @@ class Device(threading.Thread):
     def _wifi_rx_event(self, msg):
         if self._wifi_rx_callback(msg):
             data = self._device_id_str + "\tWIFI\n"
-            self.tx_zmq_pub.send_multipart([b"ALL", b'0', data.encode('utf-8')])
+            self.tx_zmq_pub.send_multipart([b"ALL", data.encode('utf-8')])
         return ""
 
     def set_dashio_callback(self, callback):
@@ -269,7 +361,7 @@ class Device(threading.Thread):
     def _dashio_rx_event(self, msg):
         if self._dashio_rx_callback(msg):
             data = self._device_id_str + "\tDASHIO\n"
-            self.tx_zmq_pub.send_multipart([b"ALL", b'0', data.encode('utf-8')])
+            self.tx_zmq_pub.send_multipart([b"ALL", data.encode('utf-8')])
         return ""
 
     def set_name_callback(self, callback):
@@ -297,7 +389,7 @@ class Device(threading.Thread):
         if name:
             self._device_name = name
             data = self._device_id_str + f"\tNAME\t{name}\n"
-            self.tx_zmq_pub.send_multipart([b"ALL", b'0', data.encode('utf-8')])
+            self.tx_zmq_pub.send_multipart([b"ALL", data.encode('utf-8')])
         return ""
 
     def set_tcp_callback(self, callback):
@@ -322,7 +414,7 @@ class Device(threading.Thread):
     def _tcp_rx_event(self, msg):
         if self._tcp_rx_callback(msg):
             data = self._device_id_str + "\tTCP\n"
-            self.tx_zmq_pub.send_multipart([b"ALL", b'0', data.encode('utf-8')])
+            self.tx_zmq_pub.send_multipart([b"ALL", data.encode('utf-8')])
         return ""
 
     def set_mqtt_callback(self, callback):
@@ -348,15 +440,29 @@ class Device(threading.Thread):
     def _mqtt_rx_event(self, msg):
         if self._mqtt_rx_callback(msg):
             data = self._device_id_str + "\tMQTT\n"
-            self.tx_zmq_pub.send_multipart([b"ALL", b'0', data.encode('utf-8')])
+            self.tx_zmq_pub.send_multipart([b"ALL", data.encode('utf-8')])
         return ""
 
-    def __init__(self,
-                 device_type: str,
-                 device_id: str,
-                 device_name: str,
-                 cfg_dict: dict = None,
-                 context=None) -> None:
+    def register_connection(self, connection):
+        """Connections registered here"""
+        if connection.zmq_connection_uuid not in self.connections_list:
+            logging.debug("DEVICE REG CONECTION")
+            self.connections_list.append(connection.zmq_connection_uuid)
+            self.rx_zmq_sub.connect(CONNECTION_PUB_URL.format(id=connection.zmq_connection_uuid))
+            connection.rx_zmq_sub.connect(CONNECTION_PUB_URL.format(id=self.zmq_connection_uuid))
+            self._send_announce()
+        if self._add_actions:
+            self.action_station.register_connection(connection)
+
+    def __init__(
+        self,
+        device_type: str,
+        device_id: str,
+        device_name: str,
+        add_actions: bool = False,
+        cfg_dict: dict = None,
+        context: zmq.Context = None
+    ) -> None:
         """DashDevice
 
         Parameters
@@ -368,14 +474,16 @@ class Device(threading.Thread):
             device_name : str
                 The name for this device
             cfg_dict : dict optional
-                Setup dict to cfgRev, defaults None
+                Setup dict to cfgRev and adds controls defined in cfg_dict, defaults None
+            add_actions : boolean
+                To include actions or not, defaults false
             context : optional
                 ZMQ context. Defaults to None.
         """
         threading.Thread.__init__(self, daemon=True)
 
-        self.zmq_pub_id = shortuuid.uuid()
-        self._b_zmq_pub_id = self.zmq_pub_id.encode('utf-8')
+        self.zmq_connection_uuid = "DVCE:" + shortuuid.uuid()
+        self._b_zmq_connection_uuid = self.zmq_connection_uuid.encode()
         self.context = context or zmq.Context.instance()
         self._wifi_rx_callback = None
         self._dashio_rx_callback = None
@@ -387,17 +495,27 @@ class Device(threading.Thread):
         self._b_device_id = self.device_id.encode('utf-8')
         self._device_name = device_name.strip()
         self._device_setup_list = []
+        self.connections_list = []
         self._device_commands_dict = {}
         self._device_commands_dict['CONNECT'] = self._make_connect
         self._device_commands_dict['STATUS'] = self._make_status
         self._device_commands_dict['CFG'] = self._make_cfg
-        self._control_dict = {}
+        self.controls_dict = {}
         self._cfg = {}
         self._device_id_str = f"\t{device_id}"
+        self._cfg["numDeviceViews"] = 0
         if cfg_dict is not None:
             self._cfg["cfgRev"] = cfg_dict['CFG']['cfgRev']
-        self._cfg["numDeviceViews"] = 0
+            self.add_all_c64_controls(cfg_dict)
+
+        self._add_actions = add_actions
+        if self._add_actions:
+            self._add_action_device_setup(True)
+            self.action_station = ActionStation(self, context=self.context)
+            time.sleep(0.5)
+            self.action_station.device_zmq_sub.connect(CONNECTION_PUB_URL.format(id=self.zmq_connection_uuid))
         self.running = True
+        time.sleep(0.5)
         self.start()
 
     def use_cfg64(self):
@@ -440,6 +558,13 @@ class Device(threading.Thread):
     def config_revision(self, val: int):
         self._cfg["cfgRev"] = val
 
+    def inc_config_revision(self):
+        """Incements the configuration revision."""
+        if "cfgRev" in self._cfg:
+            self._cfg["cfgRev"] = self._cfg["cfgRev"] + 1
+        else:
+            self._cfg["cfgRev"] = 1
+
     @property
     def device_name(self) -> str:
         """Device name
@@ -458,15 +583,24 @@ class Device(threading.Thread):
 
     def close(self):
         """Close the device"""
+        if self._add_actions:
+            self.action_station.close()
         self.running = False
 
+    def _local_command(self, msg_dict):
+        if msg_dict['msgType'] == 'send_announce':
+            self._send_announce()
+
     def run(self):
         # Continue the network loop, exit when an error occurs
 
         self.tx_zmq_pub = self.context.socket(zmq.PUB)
-        self.tx_zmq_pub.bind(DEVICE_PUB_URL.format(id=self.zmq_pub_id))
+        self.tx_zmq_pub.bind(CONNECTION_PUB_URL.format(id=self.zmq_connection_uuid))
         self.rx_zmq_sub = self.context.socket(zmq.SUB)
-        self.rx_zmq_sub.setsockopt_string(zmq.SUBSCRIBE, "COMMAND")
+        self.rx_zmq_sub.setsockopt(zmq.SUBSCRIBE, b"\tWHO")
+        self.rx_zmq_sub.setsockopt(zmq.SUBSCRIBE, b"COMMAND")
+        self.rx_zmq_sub.setsockopt(zmq.SUBSCRIBE, self._b_zmq_connection_uuid)
+        self.rx_zmq_sub.setsockopt_string(zmq.SUBSCRIBE, "\t" + self.device_id)
 
         poller = zmq.Poller()
         poller.register(self.rx_zmq_sub, zmq.POLLIN)
@@ -477,16 +611,19 @@ class Device(threading.Thread):
             except zmq.error.ContextTerminated:
                 break
             if self.rx_zmq_sub in socks:
-                msg = self.rx_zmq_sub.recv_multipart()
-                if len(msg) == 3:
-                    if msg[0] == b"COMMAND":
-                        if msg[2] == b'send_announce':
-                            self._send_announce()
-                        continue
-                    reply = self._on_message(msg[2])
-                    if reply:
-                        self.tx_zmq_pub.send_multipart([msg[0], msg[1], reply.encode('utf-8')])
-
+                try:
+                    [data, msg_from] = self.rx_zmq_sub.recv_multipart()
+                except ValueError:
+                    logging.debug("Device value error")
+                #  logging.debug("DEVICE RX: %s ,%s", msg_from, data)
+                if data == b"COMMAND":
+                    msg_dict = json.loads(msg_from)
+                    self._local_command(msg_dict)
+                    continue
+                reply = self._on_message(data)
+                if reply != "":
+                    # logging.debug("DEVICE TX: %s ,%s", msg_from, data)
+                    self.tx_zmq_pub.send_multipart([msg_from, reply.encode('utf-8')])
         self.tx_zmq_pub.close()
         self.rx_zmq_sub.close()
         self.context.term()
